import time
import hashlib
import os
import struct
from typing import List, Tuple
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.asymmetric import rsa

class UltraSecureRNG:
    def __init__(self):
        self.counter = int.from_bytes(os.urandom(32), byteorder='big')
        self.buffer = bytearray()
        self.buffer_index = 0
        self.reseed_interval = 10
        self.operations = 0
        self.seed()

    def seed(self):
        seed_data = os.urandom(4096)
        seed_data += struct.pack('>QQ', time.time_ns(), time.process_time_ns())
        seed_data += hashlib.blake2b(os.urandom(8192), digest_size=64).digest()
        self.state = hashlib.sha3_512(seed_data).digest()

    def reseed(self):
        new_seed = os.urandom(4096) + self.state
        new_seed += hashlib.sha3_512(os.urandom(8192)).digest()
        self.state = hashlib.blake2b(new_seed, digest_size=64).digest()
        self.operations = 0

    def get_random_bytes(self, n: int) -> bytes:
        if self.operations >= self.reseed_interval:
            self.reseed()

        result = bytearray()
        while len(result) < n:
            if self.buffer_index >= len(self.buffer):
                self.counter += 1
                data = self.state + self.counter.to_bytes(32, byteorder='big')
                key = os.urandom(64)
                new_bytes = hashlib.blake2b(data, digest_size=64, key=key).digest()
                self.state = hashlib.sha3_512(self.state + new_bytes + key).digest()

                hkdf = HKDF(
                    algorithm=hashes.SHA3_512(),
                    length=64,
                    salt=os.urandom(16),
                    info=b'random_generation',
                    backend=default_backend()
                )
                derived_key = hkdf.derive(new_bytes)

                cipher = Cipher(algorithms.AES(derived_key[:32]), modes.GCM(os.urandom(12)), backend=default_backend())
                encryptor = cipher.encryptor()
                self.buffer = bytearray(encryptor.update(os.urandom(64)) + encryptor.finalize())

                self.buffer_index = 0
                self.operations += 1

            remaining = n - len(result)
            chunk = self.buffer[self.buffer_index:self.buffer_index + remaining]
            result.extend(chunk)
            self.buffer_index += len(chunk)

        return bytes(result)

    def random(self) -> float:
        return int.from_bytes(self.get_random_bytes(8), byteorder='big') / (1 << 64)

def choose_random_char(character_set: str, rng: UltraSecureRNG) -> str:
    return character_set[int(rng.random() * len(character_set))]

def xor_bytes(a: bytes, b: bytes) -> bytes:
    return bytes(x ^ y for x, y in zip(a, b))

def generate_password(length: int = 32) -> str:
    if length < 20:
        raise ValueError("Password length must be at least 20 characters")

    uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    lowercase = 'abcdefghijklmnopqrstuvwxyz'
    digits = '0123456789'
    special = '!@#$%^&*()-_=+[]{}|;:,.<>?'
    extra_special = '~"\'\\/'

    all_chars = uppercase + lowercase + digits + special + extra_special
    char_groups: List[Tuple[str, int]] = [
        (uppercase, 4), (lowercase, 4), (digits, 4),
        (special, 4), (extra_special, 2)
    ]

    rng = UltraSecureRNG()

    password = []

    for group, min_count in char_groups:
        password.extend([choose_random_char(group, rng) for _ in range(min_count)])

    remaining_length = length - len(password)
    password.extend(choose_random_char(all_chars, rng) for _ in range(remaining_length))

    # Улучшенное перемешивание
    for _ in range(length * 32):
        i, j = int(rng.random() * length), int(rng.random() * length)
        password[i], password[j] = password[j], password[i]

    # Усиленное криптографическое перемешивание
    password_bytes = ''.join(password).encode()
    for _ in range(16):
        mixed = hashlib.sha3_512(password_bytes).digest()
        mixed = xor_bytes(mixed, hashlib.blake2b(password_bytes, key=os.urandom(64), salt=os.urandom(16)).digest())
        mixed = hashlib.sha3_256(mixed).digest()
        mixed = xor_bytes(mixed, hashlib.shake_256(password_bytes).digest(32))
        for i, byte in enumerate(mixed):
            j = byte % length
            password[i % length], password[j] = password[j], password[i % length]
        password_bytes = ''.join(password).encode()

    # Дополнительное преобразование с использованием RSA
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
    )
    public_key = private_key.public_key()

    encrypted = public_key.encrypt(
        password_bytes,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )

    # Финальное преобразование с использованием HKDF и ChaCha20
    hkdf = HKDF(
        algorithm=hashes.SHA3_512(),
        length=32,
        salt=os.urandom(16),
        info=b'password_finalization',
        backend=default_backend()
    )
    final_key = hkdf.derive(encrypted)

    nonce = os.urandom(16)
    algorithm = algorithms.ChaCha20(final_key, nonce)
    cipher = Cipher(algorithm, mode=None, backend=default_backend())
    encryptor = cipher.encryptor()
    final_bytes = encryptor.update(os.urandom(length))

    for i, byte in enumerate(final_bytes):
        password[i] = all_chars[byte % len(all_chars)]

    return ''.join(password)

def gen(length="32"):
    """
    Generate a secure password of the specified length.
    
    :param length: The length of the password to generate (as a string).
    :return: A secure password string.
    """
    return generate_password(int(length))

# Можно добавить дополнительные функции или классы, если нужно

# Если вы хотите, чтобы при импорте модуля были доступны определенные имена,
# вы можете указать их в __all__:
__all__ = ['gen']